@@ -7,6 +7,7 @@ import pickle
 import re
 import math
 import pandas as pd
+import numpy as np
 
 from tqdm import tqdm
 import datetime
@@ -121,26 +122,31 @@ def add_zero(nb):
 #######################################################################################################################
 
 # Put thousand separator
-def group(nb, digits=0):
+def group(nb, digits=0, unit=''):
     """Transforms a number into a string with a thousand separator.
 
     :Parameters:
         * **nb** (:obj:`float`): Number to be transformed.
         * **digits** (:obj:`int`): Number of digits to round.
+        * **unit** (:obj:`str`): Unit to be displayed (defaults to '').
 
     :Example:
         >>> pycof.group(12345)
         ... '12,345'
         >>> pycof.group(12345.54321, digits=3)
         ... '12,345.543'
+        >>> pycof.group(12.54, digits=3, unit='%')
+        ... '12.54%'
 
     :Returns:
         * :obj:`str`: Transformed number.
     """
     if math.isnan(nb):
         return('-')
+    elif nb == 0.:
+        return('-')
     else:
-        s = '%d' % nb
+        s = '%d' % round(nb, digits)
         groups = []
         if digits > 0:
             dig = '.' + str(nb).split('.')[1][:digits]
@@ -149,7 +155,7 @@ def group(nb, digits=0):
         while s and s[-1].isdigit():
             groups.append(s[-3:])
             s = s[:-3]
-        return s + ','.join(reversed(groups)) + dig
+        return s + ','.join(reversed(groups)) + dig + unit
 
 
 #######################################################################################################################
@@ -285,7 +291,7 @@ def str2bool(value):
 
 # Getting Google Calendar events
 class GoogleCalendar:
-    def __init__(self, timezone='Europe/Paris', scopes=['https://www.googleapis.com/auth/calendar.readonly']):
+    def __init__(self, timezone='Europe/Paris', scopes=['https://www.googleapis.com/auth/calendar.readonly'], temp_folder=None):
         """Get all available events on a Google Calendar.
         The `Google credentials file <https://developers.google.com/calendar/quickstart/python>`_ needs to be saved as :obj:`/etc/.pycof/google.json`.
 
@@ -293,6 +299,8 @@ class GoogleCalendar:
         :type timezone: :obj:`str`, optional
         :param scopes: Targeted permissions required. Check https://developers.google.com/calendar/auth for more details, defaults to ['https://www.googleapis.com/auth/calendar.readonly'].
         :type scopes: :obj:`list`, optional
+        :param temp_folder: Folder in which we will save the `token.pickle` authentication file, defaults to None and saves in the PYCOF temporary data folder.
+        :type temp_folder: :obj:`str`, optional
 
         :Configuration: The function requires a configuration file stored at :obj:`/etc/.pycof/google.json`.
             This file can be generated at https://developers.google.com/calendar/quickstart/python.
@@ -300,6 +308,7 @@ class GoogleCalendar:
         """
         self.timezone = pytz.timezone(timezone)
         self.scopes = scopes
+        self.data_fold = _pycof_folders('data') if temp_folder is None else temp_folder
 
     def _get_creds(self):
         """Retreive Google credentials.
@@ -311,7 +320,7 @@ class GoogleCalendar:
         # The file token.pickle stores the user's access and refresh tokens, and is
         # created automatically when the authorization flow completes for the first
         # time.
-        token_path = os.path.join(_pycof_folders('data'), 'token.pickle')
+        token_path = os.path.join(self.data_fold, 'token.pickle')
         creds_path = os.path.join(_pycof_folders('creds'), 'google.json')
 
         if os.path.exists(token_path):
@@ -479,7 +488,7 @@ def GetEmails(nb_email=1, email_address='', port=993, credentials={}):
         first_email_id = int(id_list[0])
         latest_email_id = int(id_list[-1])
 
-        df = pd.DataFrame()
+        df = []
         for num in range(latest_email_id, latest_email_id - nb_email, -1):
             typ, data = mail.fetch(str(num), '(RFC822)')
             raw_email = data[0][1]  # converts byte literal to string removing b''
@@ -496,7 +505,7 @@ def GetEmails(nb_email=1, email_address='', port=993, credentials={}):
             try:
                 _date = ddt.replace(tzinfo=datetime.timezone.utc).astimezone(tz=tz.tzlocal())
             except Exception:
-                _date = None
+                _date = np.nan
             for_df = {'From': _from, 'Subject': _subj, 'To': _to, 'Date': _date}
 
             # Get email attachments
@@ -512,8 +521,8 @@ def GetEmails(nb_email=1, email_address='', port=993, credentials={}):
                         fp.write(part.get_payload(decode=True))
                         fp.close()
 
-            df = df.append(for_df, ignore_index=True)
-        return df
+            df += [pd.DataFrame(for_df, index=[0])]
+        return pd.concat(df).reset_index(drop=True)
     except Exception as e:
         traceback.print_exc()
         print(str(e))
